# SPDX-FileCopyrightText: 2025 Ryan Lahfa <ryan.lahfa@dgnum.eu>
#
# SPDX-License-Identifier: MIT

{
  config,
  lib,
  pkgs,
  utils,
  ...
}:

let
  inherit (lib)
    optional
    mkEnableOption
    mkOption
    mkIf
    mkPackageOption
    getExe'
    getExe
    ;

  inherit (utils) escapeSystemdExecArgs genJqSecretsReplacementSnippet;

  inherit (lib.types) submodule str listOf;

  cfg = config.services.openbao;

  settingsFormat = pkgs.formats.json { };
in
{
  options = {
    services.openbao = {
      enable = mkEnableOption "OpenBao daemon";

      package = mkPackageOption pkgs "openbao" { };

      settings = mkOption {
        description = ''...'';
        type = submodule {
          freeformType = settingsFormat.type;
          options = {
            listener.tcp.address = mkOption {
              type = str;
              default = "127.0.0.1:8200";
              description = ''
                The address the OpenBao daemon will listen to.
              '';
            };
          };
        };
      };

      extraArgs = mkOption {
        type = listOf str;
        default = [ ];
        description = ''
          Additional arguments given to OpenBao
        '';
      };
    };
  };

  config = mkIf cfg.enable {
    environment.systemPackages = [ pkgs.openbao ];

    assertions = [
      {
        assertion = cfg.storageBackend == "inmem" -> (cfg.storagePath == null && cfg.storageConfig == null);
        message = ''The "inmem" storage expects no services.vault.storagePath nor services.vault.storageConfig'';
      }
      {
        assertion =
          (cfg.storageBackend == "file" -> (cfg.storagePath != null && cfg.storageConfig == null))
          && (cfg.storagePath != null -> (cfg.storageBackend == "file" || cfg.storageBackend == "raft"));
        message = ''You must set services.vault.storagePath only when using the "file" or "raft" backend'';
      }
    ];

    systemd.services.openbao = {
      description = "OpenBao server daemon";

      wantedBy = [ "multi-user.target" ];
      after = [
        "network.target"
      ] ++ optional (config.services.consul.enable && cfg.storageBackend == "consul") "consul.service";

      restartIfChanged = false; # do not restart on "nixos-rebuild switch". It would seal the storage and disrupt the clients.
      preStart = genJqSecretsReplacementSnippet (settingsFormat.generate cfg.settings "openbao-settings.json") "/var/lib/openbao/config.json";

      startLimitIntervalSec = 60;
      startLimitBurst = 3;
      serviceConfig = {
        DynamicUser = true;
        ExecStart =
          escapeSystemdExecArgs [
            (getExe cfg.package)
            "server"
            "-config"
            "/var/lib/openbao/config.json"
          ]
          ++ cfg.extraArgs;
        ExecReload = "${getExe' pkgs.coreutils "kill"} -SIGHUP $MAINPID";
        StateDirectory = "openbao";
        UMask = "0700";
        PrivateDevices = true;
        PrivateTmp = true;
        ProtectSystem = "full";
        ProtectHome = "read-only";
        AmbientCapabilities = "cap_ipc_lock";
        NoNewPrivileges = true;
        LimitCORE = 0;
        KillSignal = "SIGINT";
        TimeoutStopSec = "30s";
        Restart = "on-failure";
      };

      unitConfig.RequiresMountsFor = optional (cfg.storagePath != null) cfg.storagePath;
    };
  };

}
